@@ -9,6 +9,8 @@ use Carp;
 use Exporter::Tidy default => [
     qw($bytes $BYTES
       $utf8 $UTF8
+      $data1 $DATA1
+      $data2 $DATA2
       enc_ok
       enc_error_ok
       decod_ok
@@ -28,6 +30,40 @@ our $bytes = pack( 's<', 255 );
 my $bytes_length = bytes::length($bytes);
 our $BYTES = 'B' . $bytes_length . ':' . $bytes;
 
+our $data1 = {
+    bools   => [ $Bifcode::FALSE, $Bifcode::TRUE, ],
+    bytes   => \$bytes,
+    integer => 25,
+    float   => 1.0 / 8.0e9,
+    undef   => undef,
+    utf8    => $utf8,
+};
+
+our $DATA1 = '{'
+  . ( 'U5:bools' . '[01]' )
+  . ( 'U5:bytes' . $BYTES )
+  . ( 'U5:float' . 'F1.25e-10,' )
+  . ( 'U7:integer' . 'I25,' )
+  . ( 'U5:undef' . '~' )
+  . ( 'U4:utf8' . $UTF8 ) . '}';
+
+our $data2 = {
+    bools   => [ $Bifcode::FALSE, $Bifcode::TRUE, ],
+    bytes   => \$bytes,
+    integer => 24,
+    float   => 1.0 / 8.0e8,
+    undef   => undef,
+    utf8    => $utf8,
+};
+
+our $DATA2 = '{'
+  . ( 'U5:bools' . '[01]' )
+  . ( 'U5:bytes' . $BYTES )
+  . ( 'U5:float' . 'F1.25e-9,' )
+  . ( 'U7:integer' . 'I24,' )
+  . ( 'U5:undef' . '~' )
+  . ( 'U4:utf8' . $UTF8 ) . '}';
+
 sub enc_ok {
     croak 'usage: enc_ok($1,$2)'
       unless 2 == @_;
